// test account (avicious.406+test@gmail.com) API key
// eZbV6TDcl2qUmtEAMLllN8qOHeb1hQiS8sVEoG2OmfqLym4UqihiFL83xgM2SLGPtz8HS2DeX45sk89B
// test offer id: https://hhtestnet.com/offers/F8W2srPJK2JKwYdh
// TODO: unmarshal json (encoding/json)
package main

import (
	"fmt"
	"log"
	"os"
	"flag"
	"net/http"
	"io/ioutil"
	"encoding/json"
	"errors"
	"gosrc.io/xmpp"
	"gosrc.io/xmpp/stanza"
	"github.com/spf13/pflag"
	"github.com/spf13/viper"
	"strings"
	"strconv"
	"path"
	"encoding/xml"
)

const (
	// APIEndpoint = "https://hodlhodl.com/api/v1"
	TestAPIEndpoint = "https://hhtestnet.com/api/v1"
	APIkey = "***REMOVED***"
	
	// Config
	infoFormat = "====== "
	defaultConfigFilePath = "./"
	configFileName = "config"
	configType     = "yaml"
	logStanzasOn   = "logger_on"
	logFilePath    = "logfile_path"

	//Keys in config
	serverAddressKey = "full_address"
	clientJid        = "jid"
	clientPass       = "pass"
	configContactSep = ";"
)

var (
	CorrespChan = make(chan string, 1)
	textChan    = make(chan string, 5)
	rawTextChan = make(chan string, 5)
	killChan    = make(chan error, 1)
	errChan     = make(chan error)
	rosterChan  = make(chan struct{})

	logger        *log.Logger
	disconnectErr = errors.New("disconnecting client")
)

// Notification export
type Notification struct {
	Status        string `json:"status"`
	Notifications []struct {
		ID    string `json:"id"`
		Title string `json:"title"`
		Body  string `json:"body"`
		Link  string `json:"link"`
	} `json:"notifications"`
}

type config struct {
	Server     map[string]string `mapstructure:"server"`
	Client     map[string]string `mapstructure:"client"`
	Contacts   string            `string:"contact"`
	LogStanzas map[string]string `mapstructure:"logstanzas"`
} 



func main() {
	//body:= strings.NewReader(' -X POST -H "Authorization: Bearer ***REMOVED***" -H "Content-Type: application/json"`)
	

	// ============================================================
	// Parse the flag with the config directory path as argument
	flag.String("c", defaultConfigFilePath, "Provide a path to the directory that contains the configuration"+
		" file you want to use. Config file should be named \"config\" and be in YAML format..")
	pflag.CommandLine.AddGoFlagSet(flag.CommandLine)
	pflag.Parse()

	// ==========================
	// Read configuration
	c := readConfig()

	// Setup logger
	on, err := strconv.ParseBool(c.LogStanzas[logStanzasOn])
	if err != nil {
		log.Panicln(err)
	}
	if on {
		f, err := os.OpenFile(path.Join(c.LogStanzas[logFilePath], "logs.txt"), os.O_RDWR|os.O_APPEND|os.O_CREATE, 0666)
		if err != nil {
			log.Panicln(err)
		}
		logger = log.New(f, "", log.Lshortfile|log.Ldate|log.Ltime)
		logger.SetOutput(f)
		defer f.Close()
	}

	// ==========================
	// Run TUI
	//================================
	startClient(c)

}

func startClient(config *config) {

	// Client Setup
	clientCfg := &xmpp.Config {
		TransportConfiguration: xmpp.TransportConfiguration{
			Address: config.Server[serverAddressKey],
		},
		Jid: config.Client[clientJid],
		Credential: xmpp.Password(config.Client[clientPass]),
		Insecure: true,
	}


	var err error
	var client *xmpp.Client
	router := xmpp.NewRouter()
	
	handleMessage := func(s xmpp.Sender, p stanza.Packet) {
		msg, ok := p.(stanza.Message)
		if !ok {
			_, _ = fmt.Fprintf(os.Stdout, "Ignoring packet: %T\n", p)
			return
		}
	
		_, _ = fmt.Fprintf(os.Stdout, "Body = %s - from = %s\n", msg.Body, msg.From)
		reply := stanza.Message{Attrs: stanza.Attrs{To: msg.From}, Body: msg.Body}
		_ = s.Send(reply)
	}

	errorHandler:= func(err error) {
		fmt.Println(err.Error())
	}

	router.HandleFunc("message", handleMessage)
	if client, err = xmpp.NewClient(clientCfg, router, errorHandler); err != nil {
		log.Panicln(fmt.Sprintf("Could not create a new client ! %s", err))
	}

	// Client connection
	if err = client.Connect(); err != nil {
		fmt.Sprintf("%sXMPP connection failed: %s", infoFormat, err)
		fmt.Println("Failed to connect to server. Exiting...")
		servConnFail := errors.New("failed to connect to server. Check your configuration ? Exiting")
		errChan <- servConnFail
		return
	} else {
		fmt.Println("Client is running")
	}

	/*client, err := xmpp.NewClient(clientCfg, router, errorHandler)
	if err != nil {
		log.Panicln(fmt.Sprintf("Could not create a new client ! %s", err))
		//log.Fatalf("%+v", err)
	} else {
		fmt.Println("Client running....")
	}*/

	//Connection manager, reconect automatically
	cm := xmpp.NewStreamManager(client, nil)
	log.Fatal(cm.Run())


	// ====================
	// Start working
	currentContacts := strings.Split(config.Contacts, configContactSep)[0]
	fmt.Println(infoFormat+"Now sending messages to "+currentContacts+" in a private conversation\n")
	CorrespChan <- currentContacts
	fmt.Println("currentContacts", currentContacts)

}



func startMessaging(client xmpp.Sender, config *config) {
	var text string
	var correspondent string
	for {
		select {
		case err := <-killChan:
			if err == disconnectErr {
				sc := client.(xmpp.StreamClient)
				sc.Disconnect()
			} else {
				logger.Println(err)
			}
			return
		case text = <-textChan:
			reply := stanza.Message{Attrs: stanza.Attrs{To: correspondent, Type: stanza.MessageTypeChat}, Body: text}
			if logger != nil {
				raw, _ := xml.Marshal(reply)
				logger.Println(string(raw))
			}
			err := client.Send(reply)
			if err != nil {
				fmt.Printf("There was a problem sending the message : %v", reply)
				return
			}
		case text = <-rawTextChan:
			if logger != nil {
				logger.Println(text)
			}
			err := client.SendRaw(text)
			if err != nil {
				fmt.Printf("There was a problem sending the message : %v", text)
				return
			}
		case crrsp := <-CorrespChan:
			correspondent = crrsp
		case <-rosterChan:
			fmt.Println("rosterchan")
		}

	}
}

func handleMessage(s xmpp.Sender, p stanza.Packet) {
	msg, ok := p.(stanza.Message)
	if !ok {
		_, _ = fmt.Fprintf(os.Stdout, "Ignoring packet: %T\n", p)
		return
	}

	_, _ = fmt.Fprintf(os.Stdout, "Body = %s - from = %s\n", msg.Body, msg.From)
	reply := stanza.Message{Attrs: stanza.Attrs{To: msg.From}, Body: msg.Body}
	_ = s.Send(reply)
}



func gethdlNotif() {

	req, err := http.NewRequest("POST", TestAPIEndpoint + "/notifications/read", nil)
	if err != nil {
		fmt.Println(err)
	}
	req.Header.Add("Authorization", "Bearer " + APIkey)
	req.Header.Set("Content-Type", "application/json")

	resp, err := http.DefaultClient.Do(req)
	if err!= nil {
		fmt.Println(err)
	}

	fmt.Println(resp.Status)
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		fmt.Println(err)
	} else {
		res:= Notification{}
		json.Unmarshal([]byte(body), &res)
		fmt.Println("RESULT", res)
		fmt.Println("RESULTAT", string(body))
	}


	defer resp.Body.Close()

}




func readConfig() *config {
	viper.SetConfigName(configFileName) // name of config file (without extension)
	viper.BindPFlags(pflag.CommandLine)
	viper.AddConfigPath(viper.GetString("c")) // path to look for the config file in
	err := viper.ReadInConfig()               // Find and read the config file
	if err := viper.ReadInConfig(); err != nil {
		if _, ok := err.(viper.ConfigFileNotFoundError); ok {
			log.Fatalf("%s %s", err, "Please make sure you give a path to the directory of the config and not to the config itself.")
		} else {
			log.Panicln(err)
		}
	}

	viper.SetConfigType(configType)
	var config config
	err = viper.Unmarshal(&config)
	if err != nil {
		panic(fmt.Errorf("Unable to decode Config: %s \n", err))
	}

	// Check if we have contacts to message
	if len(strings.TrimSpace(config.Contacts)) == 0 {
		log.Panicln("You appear to have no contacts to message !")
	}
	// Check logging
	config.LogStanzas[logFilePath] = path.Clean(config.LogStanzas[logFilePath])
	on, err := strconv.ParseBool(config.LogStanzas[logStanzasOn])
	if err != nil {
		log.Panicln(err)
	}
	if d, e := isDirectory(config.LogStanzas[logFilePath]); (e != nil || !d) && on {
		log.Panicln("The log file path could not be found or is not a directory.")
	}

	return &config
}

// If an error occurs, this is used to kill the client
func errorHandler(err error) {
	killChan <- err
}


func isDirectory(path string) (bool, error) {
	fileInfo, err := os.Stat(path)
	if err != nil {
		return false, err
	}
	return fileInfo.IsDir(), err
}

